-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
-- Counter_mod16 prototype in LAB7 for adapting the application to a DE10-Lite
-- CLK_Generator chip
-- Strategy: Schematic using frequency dividers and T_FF to square signals
---------------------------------------------------
-- Project P8. Lecture L8.2 on designing CLK_generators  
-- https://digsys.upc.edu/csd/P08/L08_2/L8_2.html

-- Reference and schematic at: 
-- https://digsys.upc.edu/csd/P07/Counter_mod16/Counter_mod16.html#prototyping
--------------------------------------------------------------------------------
-- OSC_CLK_in = 50 MHz  from the DE10-Lite oscillator
-- To run the Counter_mod16 with two CLK frequencies:
-- CLK_16Hz_SQ
-- CLK_1Hz_SQ 
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY CLK_Generator IS  
      PORT(	
			OSC_CLK_in	:	IN		STD_LOGIC;
			CD 			:	IN		STD_LOGIC;
			CE			:	IN		STD_LOGIC;
			CLK_16Hz_SQ	:	OUT	STD_LOGIC;
			CLK_1Hz_SQ	:	OUT	STD_LOGIC			
		);
END CLK_Generator;

ARCHITECTURE schematic OF CLK_Generator  IS  
-- Components
	COMPONENT Freq_Div_1562500	IS                 
      PORT(CD,CLK,CE		: 	IN  std_logic;
           TC1562500		:	OUT std_logic
          );
	END COMPONENT;
	
	COMPONENT Freq_Div_16	IS                 
		PORT(	
			CD,CLK,CE		: 	IN  std_logic;
			TC16			:	OUT std_logic
		);
	END COMPONENT;		
	
	COMPONENT T_FF IS
		Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T 		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL CLK			:	std_logic;
	SIGNAL Pulsed_32Hz	:	std_logic;
	SIGNAL Pulsed_2Hz	:	std_logic;
		
	BEGIN  
-- Instantiation of components

	Chip1	: 	Freq_Div_1562500
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK			=> CLK,
			CD 			=> CD,
			CE 			=> CE,
			TC1562500	=> Pulsed_32Hz					  
		);
	
	Chip2	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_32Hz,
			Q 		=> CLK_16Hz_SQ				  
		);
		
	Chip3	: 	Freq_Div_16
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK			=> CLK,
			CD 			=> CD,
			CE 			=> Pulsed_32Hz,
			TC16		=> Pulsed_2Hz					  
		);	

	Chip4	: T_FF
		PORT MAP (
-- from component name 	=> to signal or port name	
			CLK			=> CLK,
			CD 			=> CD,
			T			=> Pulsed_2Hz,
			Q 			=> CLK_1Hz_SQ				  
		);		
		
-- Connections and logic between components

	-- The internal CLK form the external quartz crystal 
	CLK <= OSC_CLK_in;
	
END schematic ;
